from astropy.units import Quantity, uV  # pyright: ignore[reportAttributeAccessIssue]
from astropy.units.physical import time
from gwpy.time import Time

from ..metadata.scatterer import Scatterer as MetadataScatterer
from ..scatterer import Scatterer
from ..utils import download_timeseries


class ScattererFactory:
    """
    Factory class to create Component
    """

    @staticmethod
    def fetch(
        start_time: Time,
        duration: Quantity[time],
        metadata: MetadataScatterer,
        force_virgotools: bool = False,
    ) -> Scatterer:
        """
        Fetch a component from online data

        Construct a Component object, containing all the data of
        this component
        Parameters
        ----------
        start_time : gwpy.Time
            Starting time
        duration : Quantity[time]
            Duration
        metadata : MetadataComponent
            Component's metadata
        force_virgotools : bool
            Force the use of the virgotools library in place of gwpy
            first to be faster. Default to False
        Returns
        -------
        Scatterer
        """
        sensor = download_timeseries(
            channel="V1:{}_LC_Z".format(metadata.name),
            start_time=start_time,
            duration=duration,
            unit=uV,  # in um, but the calibration factor is considered as m/V
            force_virgotools=force_virgotools,
        )
        return Scatterer(metadata=metadata, sensor=sensor)
