from typing import final
from h5py import File, Group

from .section import Section
from .measurement import Measurement
from .metadata.root import Root as Metadata


@final
class Root(Section):
    """
    Root object containing multiple measurement. It also can be called a
    package.
    """

    def __init__(
        self, metadata: Metadata, measurements: list[Measurement]
    ):
        """
        Parameters
        ----------
        metadata : bed.MetadataRoot
            Metadata associated to the data
        measurements : list[bed.Measurement]
            Measurements associated to the package
        """
        self.metadata = metadata
        self.measurements = measurements

    @staticmethod
    def from_hdf5(store: File | Group) -> "Root":
        measurements_group = store.get("measurements")
        if not isinstance(measurements_group, Group):
            raise ValueError(
                "measurements group should exists, and not be {}".format(
                    type(measurements_group)
                )
            )
        measurements = measurements_group.values()
        return Root(
            Metadata.from_hdf5(store),
            [
                Measurement.from_hdf5(measurement)
                for measurement in measurements
            ],
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        _ = self.metadata.to_hdf5(store)
        for measurement in self.measurements:
            _ = measurement.to_hdf5(
                store.require_group("measurements").create_group(
                    measurement.metadata.name,
                )
            )
        return store
